'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const consumerFactory = function consumerFactory(args) {
  const channel = args.channel,
        queue = args.queue,
        subscriber = args.subscriber,
        options = args.options;

  var _ref = options || {};

  const noAck = _ref.noAck;


  return (() => {
    var _ref2 = (0, _asyncToGenerator3.default)(function* (message) {
      try {
        const content = message.content,
              properties = message.properties;
        const contentType = properties.contentType,
              contentEncoding = properties.contentEncoding;


        let payload = content.toString(contentEncoding);

        if (contentType === 'application/json') {
          payload = JSON.parse(payload);
        }

        const consumerObj = {
          raw: message,
          properties,
          payload,
          channel,
          queue
        };

        const resultCode = yield subscriber(consumerObj);

        // Default assumes the reply was handled manually
        if (resultCode && !noAck) {
          if (resultCode === _constants.ACK) {
            channel.ack(message);
          } else if (resultCode === _constants.NACK) {
            channel.nack(message);
          }
        }
      } catch (error) {
        if (!noAck) {
          channel.nack(message);
        }
        throw error;
      }
    });

    return function (_x) {
      return _ref2.apply(this, arguments);
    };
  })();
};

exports.default = consumerFactory;